<?php

abstract class emailTypes
{
    const PERSONAL = 'personal';
    const BUSINESS = 'business';
    const SCHOOL = 'school';
    const PRIMARY = 'primary_address';
}

class emailRelationships
{
    private $emailTypes = array();
    private $relationships = array();
    private $bean_name = null;
    private $bean_id;

    /**
     * Returns the boolean value of the type
     * @param string $type Must be an emailTypes const
     * @return bool
     */
    public function getEmailAddressOfType($type)
    {
        $email = $this->emailTypes[$type];
         return $this->emailTypes[$type];
    }

    /**
     * This will flip the type from the emailRelationship that currently has that type to 
     * an existing or new emailRelationship.
     * Then it will audit the change.
     * @param string $type Must be an emailTypes const
     * @param string $email_address
     */
    function setEmailAddressType($type, $email_address, $current_user)
    {
        $newEmailType = $this->getEmailAddressInfo($email_address);  // Get the emailRelationship it currently has
        $oldEmailType = $this->relationships[$this->emailTypes[$type]];  // Get the relationship that currently has that type.

        if(is_null($newEmailType)) {
            $newEmailType_get_email_address_caps = '';
            $newEmailType_get_email_address = '';
        }
        else {
            $newEmailType_get_email_address_caps =  $newEmailType->get_email_address_caps();
            $newEmailType_get_email_address =  $newEmailType->get_email_address();
        }


        if(is_null($oldEmailType)) {
            $oldEmailType_get_email_address_caps = '';
            $oldEmailType_get_email_address = '';
        }
        else {
            $oldEmailType_get_email_address_caps =  $oldEmailType->get_email_address_caps();
            $oldEmailType_get_email_address =  $oldEmailType->get_email_address();
        }
        if($type != '') {
             if ($newEmailType_get_email_address_caps != $oldEmailType_get_email_address_caps) {
                 if (!is_null($oldEmailType)) {
                     $oldEmailType->setType($type, 0); // Makes change to contact_cstm or email_addr_bean_rel
                        if ($oldEmailType->getType($type) == 0) // Makes sure change was successful
                        {
                            $this->emailTypes[$type] = null; 
                        }
                    }
                 if(!is_null($newEmailType)) {
                     $newEmailType->setType($type, 1); // Makes change to contact_cstm or email_addr_bean_rel
                     if ($newEmailType->getType($type) == 1) // Makes sure change was successful
                     {
                         $this->emailTypes[$type] = $newEmailType; 
                     }

                 }
                $this->auditChange($type, $oldEmailType_get_email_address, $newEmailType_get_email_address, $current_user);
                // Makes changes to the contact_audit table.
            }
         }
    }

    /*
     * Adds email change to contact_audit so that people can see the change in the audit report.
     * @param $type is one of the const in emailTypes abstract class.
     * @param $oldEmailAddress
     * @param $newEmailAddress
     * @param $current_user
     */
    private function auditChange( $type, $oldEmailAddress = '', $newEmailAddress = '', $current_user)
    {

        $labelName = null;
        switch($type)
        {
            case emailTypes::BUSINESS : $labelName = 'business_email_c'; break;
            case emailTypes::SCHOOL: $labelName = 'school_email_c'; break;
            case emailTypes::PERSONAL: $labelName = 'personal_email_c'; break;
            case emailTypes::PRIMARY : $labelName = 'email1'; break;
        }
        $tableName = null;
        switch($this->bean_name)
        {
            case 'Contacts': $tableName = 'contacts_audit'; break;
        }
        $id = create_guid();
        $date_created = TimeDate::getInstance();

        $enterAudit = "INSERT INTO `sugarcrm_test`.`contacts_audit` (
        `id`, `parent_id`, `date_created`, `created_by`, `field_name`, `data_type`, `before_value_string`,
        `after_value_string`)
        VALUES
        ({$id},
        {$this->bean_id},
        {$date_created},
        {$current_user},
        {$labelName},
        'varchar',
        {$oldEmailAddress},
        {$newEmailAddress});";
    }

    /**
     * Takes the email  address and find or creates a relationship with the contact.
     * @param string $email_address
     * @return emailRelationship 
    */
    function getEmailAddressInfo($email_address)
    {
        if($email_address != '')
        {

            $email_address_caps = strtoupper($email_address);
            // If emailrelationship is not in $this->relationship  then create new relationship.
	        if(!isset($this->relationships[$email_address_caps])) 
            {
                $emailAddressInfo = new emailRelationship($email_address, $this->bean_id, $this->bean_name);
                $this->relationships[$email_address_caps] = $emailAddressInfo;
            }
            // else get the relationship from the $this->relationship array.
	        else
            {
		        $emailAddressInfo = $this->relationships[$email_address_caps];
            }

        }
        else
        {
            $emailAddressInfo = null;
        }
        return $emailAddressInfo;
    }

    /**
     * @param string $bean_name
     * @param string $bean_id
     */
    function __construct($bean_name, $bean_id)
    {
        global $db;
        $this->db = $db;
        $this->bean_id = $bean_id;
        $this->bean_name = $bean_name;
        $this->getCurrentRelatedEmailAddresses_FromDB(); // fills the $this->relationship array with emailRelationship objects.
    }

    /**
     * Gets the Email Relationships from the database.
     * Rewrote Amy Cox 11-18-2020
     */
    private function getCurrentRelatedEmailAddresses_FromDB()
    {

    $emailRelationshipQuery = <<<SQL
        SELECT email_addr_bean_rel.id as email_addr_bean_rel_id,
            email_address_id,
            email_address_caps,
            email_address,
            if(upper(contacts_cstm.personal_email_c) = email_address_caps, 1, 0) as personal,
            if(upper(contacts_cstm.school_email_c) = email_address_caps, 1, 0) as school,
            if(upper(contacts_cstm.business_email_c) = email_address_caps, 1, 0) as business
        FROM email_addr_bean_rel
        INNER JOIN email_addresses 
	        ON email_addr_bean_rel.email_address_id = email_addresses.id
        INNER JOIN contacts_cstm 
	        ON contacts_cstm.id_c = email_addr_bean_rel.bean_id
        WHERE email_addr_bean_rel.bean_id = @contact_id;
SQL;

        try {
            $emailRelationship = $this->db->query($emailRelationshipQuery);
            while ($row = $this->db->fetchByAssoc($emailRelationship)) {

                $relationship = new emailRelationship($row, $this->bean_id, $this->bean_name);
                $email_address_caps = $row['email_address_caps'];


                $this->relationships[$email_address_caps] = $relationship;
                $relationship_get_email_address_caps = $relationship->get_email_address_caps();

                if($relationship->getType(emailTypes::PERSONAL) == 1)
                {

                    $this->emailTypes[emailTypes::PERSONAL] = $relationship_get_email_address_caps;
                }
                if($relationship->getType(emailTypes::SCHOOL) == 1)
                {

                    $this->emailTypes[emailTypes::SCHOOL] = $relationship_get_email_address_caps;
                }
                if($relationship->getType(emailTypes::BUSINESS) == 1)
                {

                    $this->emailTypes[emailTypes::BUSINESS] = $relationship_get_email_address_caps;

                }
                if($relationship->getType(emailTypes::PRIMARY) == 1)
                {

                    $this->emailTypes[emailTypes::PRIMARY] = $relationship_get_email_address_caps;

                }

            }

        } catch (Exception $e) {
            $GLOBALS['log']->fatal(__FILE__ . " - " . __CLASS__ . "->" . __FUNCTION__ . "Error getting emails related to a {$this->bean_name} {$e->getMessage()}");
        }


    }

}

class emailRelationship
{
    private $email_addr_bean_rel_id;
    private $email_address_id;
    private $email_address_caps;
    private $email_address;
    private $school;
    private $personal;
    private $business;
    private $primary_address;
    private $timedate;
    private $bean_name;
    private $bean_id;

    function get_email_address() { return trim($this->email_address); }
    function get_email_address_caps() { return trim($this->email_address_caps); }
    function getType($typeName)
    {

        $emailType = null;
        switch($typeName){
            case emailTypes::SCHOOL: $typeValue =  $this->school; break;
            case emailTypes::PERSONAL: $typeValue =  $this->personal; break;
            case emailTypes::BUSINESS: $typeValue =  $this->business; break;
            case emailTypes::PRIMARY: $typeValue =  $this->primary_address; break;
        }

        return $typeValue;
    }

    /*
     * 
     */
    function setType($typeName, $typeValue)
    {

            try {

                switch($typeName){
                    case emailTypes::SCHOOL:
                      $this->school = $typeValue;
                        $updateFlag = "UPDATE contacts_cstm SET school_email_c = '{$this->email_address}'";
                        break;
                    case emailTypes::PERSONAL:
                        $this->personal = $typeValue;
                        $updateFlag = "UPDATE contacts_cstm SET personal_email_c = '{$this->email_address}'";
                        break;
                    case emailTypes::BUSINESS:
                        $this->business = $typeValue;
                        $updateFlag = "UPDATE contacts_cstm SET business_email_c = '{$this->email_address}'";
                    break;
                    case emailTypes::PRIMARY:
                        $this->primary_address = $typeValue;
                        $updateFlag = "UPDATE email_addr_bean_rel SET {$typeName} = {$typeValue} WHERE id = '{$this->email_addr_bean_rel_id}'";
                        
                        break;
                }

                $this->db->query($updateFlag);

            } catch (Exception $e) {
                $GLOBALS['log']->fatal(__FILE__ . " - " . __CLASS__ . "->" . __FUNCTION__ . "Error Removing the [$this->type} flag from an email relationship " . $e->getMessage());
            }

    }


    function __construct($input, $bean_id = '', $bean_name = '')
    {

        global $db;
        $this->db = $db;



        if(is_array($input))
        {

            $row = $input;
            $this->email_address = trim($row['email_address']);
            $this->email_address_caps = trim($row['email_address_caps']);
            $this->email_address_id = $row['email_address_id'];
            $this->email_addr_bean_rel_id = $row['email_addr_bean_rel_id'];
            $this->primary_address = $row['primary_address'];
            $this->school = $row['school'];
            $this->personal = $row['personal'];
            $this->business = $row['business'];
            $this->bean_id = $bean_id;
            $this->bean_name = $bean_name;
        }
        else
        {

            $this->timedate = TimeDate::getInstance();
            $this->email_address = trim($input);
            $this->bean_id = $bean_id;
            $this->bean_name = $bean_name;
            $this->email_address_caps = strtoupper(trim($input));
            $this->email_address_id = $this->getEmailAddressId_FromDB(trim($this->email_address_caps));

            if(is_null($this->email_address_id) || $this->email_address_id == '')
            {
                $this->email_address_id = $this->insertNewEmailAddress(); // Makes change to email_addresses table
            }
            $this->insertNewRelationship(); // Makes changes to email_addr_bean_rel table

        }

    }


    /**
     * Creates an Email Address in the email_addresses table.
     * @return String
     */
    private function insertNewEmailAddress()
    {
        
       // The following trims should not be needed, but are here just in case.
        $this->email_address = trim($this->email_address);
        $this->email_address_caps = trim($this->email_address_caps);

        $email_address_id = create_guid();
        $insertEmailAddressQuery = "INSERT INTO email_addresses (id, email_address, email_address_caps , date_created, date_modified)";
        $insertEmailAddressQuery .= "VALUES ('{$email_address_id}', '{$this->email_address}', '{$this->email_address_caps}', '{$this->timedate->nowDb()}','{$this->timedate->nowDb()}')";
     
    

        try {
            $this->db->query($insertEmailAddressQuery);
            $this->email_address_id = $email_address_id;
        } catch (Exception $e) {
            $GLOBALS['log']->fatal(__FILE__ . " - " . __CLASS__ . "->" . __FUNCTION__ . "Error Inserting a email address into email_address table " . $e->getMessage());
        }
        return $email_address_id;
    }

    /**
     * Retrieves the GUID of an Email Address.
     * @param $email_address_caps
     * @return mixed
     */
    private function getEmailAddressId_FromDB()
    {
        // The following trim should not be needed, but is here just in case.
        $this->email_address_caps = trim($this->email_address_caps);

        $email_address_id = null;


        $findEmailAddressQuery = "SELECT id from email_addresses WHERE deleted = 0 AND email_address_caps = '{$this->email_address_caps}'";
        try {
            $email_address_id = $this->db->getOne($findEmailAddressQuery);

        } catch (Exception $e) {
            $GLOBALS['log']->fatal(__FILE__ . " - " . __CLASS__ . "->" . __FUNCTION__ . "Error Querying for email address in emailaddresses table " . $e->getMessage());
        }

        return $email_address_id;
    }

    /**
     * 4/7/2015 3:28 PM
     * Create a new Email Address Relationship to the Contact.
     * @param $email_address_id
     * @return String
     */
    private function insertNewRelationship()
    {


        // Create Relationship
        $email_addr_bean_rel_id = create_guid();


        /*Insert related bean for email*/
        $insertRelationshipBetweenContactAndEmailAddressQuery = "INSERT INTO email_addr_bean_rel (id, email_address_id, bean_id, bean_module, primary_address, date_created, date_modified)";
        $insertRelationshipBetweenContactAndEmailAddressQuery .= "VALUES ('{$email_addr_bean_rel_id}', '{$this->email_address_id}', '{$this->bean_id}', '{$this->bean_name}', 0, '{$this->timedate->nowDb()}','{$this->timedate->nowDb()}')";
        try {
            $this->db->query($insertRelationshipBetweenContactAndEmailAddressQuery);
            $this->email_addr_bean_rel_id = $email_addr_bean_rel_id;
        } catch (Exception $e) {
            $GLOBALS['log']->fatal(__FILE__ . " - " . __CLASS__ . "->" . __FUNCTION__ . "Error Inserting a relationship into the email_addr_bean_rel table  " . $e->getMessage());
        }


    }
}



